#include "tmesh.h"
#include "linesegment.h"
#include <float.h>

TMesh::TMesh(V3 C, V3 dims) {

  vertsN = 8;
  trisN = 12;
  
  verts = new V3[vertsN];
  tris = new unsigned int[trisN*3];

  int i = 0;
  verts[i] = C + V3(-dims[0]/2.0f, dims[1]/2.0f, dims[2]/2.0f);
  i++;
  verts[i] = C + V3(-dims[0]/2.0f, -dims[1]/2.0f, dims[2]/2.0f);
  i++;
  verts[i] = C + V3(+dims[0]/2.0f, -dims[1]/2.0f, dims[2]/2.0f);
  i++;
  verts[i] = C + V3(+dims[0]/2.0f, +dims[1]/2.0f, dims[2]/2.0f);
  i++;
  verts[i] = C + V3(-dims[0]/2.0f, dims[1]/2.0f, -dims[2]/2.0f);
  i++;
  verts[i] = C + V3(-dims[0]/2.0f, -dims[1]/2.0f, -dims[2]/2.0f);
  i++;
  verts[i] = C + V3(+dims[0]/2.0f, -dims[1]/2.0f, -dims[2]/2.0f);
  i++;
  verts[i] = C + V3(+dims[0]/2.0f, +dims[1]/2.0f, -dims[2]/2.0f);
  i++;

  i = 0;
  tris[i*3+0] = 0;
  tris[i*3+1] = 1;
  tris[i*3+2] = 2;
  i++;
  tris[i*3+0] = 0;
  tris[i*3+1] = 2;
  tris[i*3+2] = 3;
  i++;
  tris[i*3+0] = 3;
  tris[i*3+1] = 2;
  tris[i*3+2] = 6;
  i++;
  tris[i*3+0] = 3;
  tris[i*3+1] = 6;
  tris[i*3+2] = 7;
  i++;
  tris[i*3+0] = 7;
  tris[i*3+1] = 6;
  tris[i*3+2] = 5;
  i++;
  tris[i*3+0] = 7;
  tris[i*3+1] = 5;
  tris[i*3+2] = 4;
  i++;
  tris[i*3+0] = 4;
  tris[i*3+1] = 5;
  tris[i*3+2] = 1;
  i++;
  tris[i*3+0] = 4;
  tris[i*3+1] = 1;
  tris[i*3+2] = 0;
  i++;
  tris[i*3+0] = 0;
  tris[i*3+1] = 3;
  tris[i*3+2] = 7;
  i++;
  tris[i*3+0] = 0;
  tris[i*3+1] = 7;
  tris[i*3+2] = 4;
  i++;
  tris[i*3+0] = 1;
  tris[i*3+1] = 5;
  tris[i*3+2] = 6;
  i++;
  tris[i*3+0] = 1;
  tris[i*3+1] = 6;
  tris[i*3+2] = 2;
  i++;

}

void TMesh::Project(PPC *ppc, V3 *&projVerts) {

  projVerts = new V3[vertsN];
  for (int i = 0; i < vertsN; i++) {
    ppc->Project(verts[i], projVerts[i]);
  }

}


void TMesh::RenderPoints(int ptSize, unsigned int color, 
  PPC *ppc, FrameBuffer *fb) {

  V3 *projVerts;
  Project(ppc, projVerts);

  for (int i = 0; i < vertsN; i++) {
    if (projVerts[i][0] == FLT_MAX)
      continue;
    fb->DrawPoint(projVerts[i], ptSize, color);
  }

  delete projVerts;

}

void TMesh::RenderWF(unsigned int color, 
  PPC *ppc, FrameBuffer *fb) {

  V3 *projVerts;
  Project(ppc, projVerts);

  for (int i = 0; i < trisN; i++) {
    int inds[3];
    inds[0] = tris[3*i+0];
    inds[1] = tris[3*i+1];
    inds[2] = tris[3*i+2];
    for (int j = 0; j < 3; j++) {
      int _j = (j+1)%3;
      LineSegment ls(projVerts[inds[j]], projVerts[inds[_j]]);
      ls.Draw2D(fb, color);
    }
  }

  delete projVerts;

}
